@@ -115,7 +115,11 @@ function sourceend() {
 	if [ -n "$testpath" ]; then
 		if [ $INVALIDCODE -eq 1 ]; then
 			PACKAGEFLAGS=$([ -z "$PACKAGES" ] || echo $PACKAGES | xargs -n 1 echo -n " --pkg")
-			echo "! $VALAC $VALAFLAGS $PACKAGEFLAGS -C $SOURCEFILE" > check
+			echo '' > prepare
+			echo "G_DEBUG=fatal-warnings $VALAC $VALAFLAGS $PACKAGEFLAGS -C $SOURCEFILE" > check
+			echo "RET=\$?" >> check
+			echo "if [ \$RET -ne 1 ]; then exit 1; fi" >> check
+			echo "exit 0" >> check
 		elif [ $GIRTEST -eq 1 ]; then
 			if [ $PART -eq 1 ]; then
 				echo "  </namespace>" >> $SOURCEFILE
